#
# Copyright (c) 2023 Airbyte, Inc., all rights reserved.
#

from dataclasses import InitVar, dataclass
from typing import Any, List, Mapping, Optional, Union

import requests
from airbyte_cdk.sources.declarative.interpolation import InterpolatedString
from airbyte_cdk.sources.declarative.requesters.paginators.strategies.pagination_strategy import PaginationStrategy
from airbyte_cdk.sources.declarative.types import Config


@dataclass
class OffsetIncrement(PaginationStrategy):
    """
    Pagination strategy that returns the number of records reads so far and returns it as the next page token
    Examples:
        # page_size to be a constant integer value
        pagination_strategy:
          type: OffsetIncrement
          page_size: 2

        # page_size to be a constant string value
        pagination_strategy:
          type: OffsetIncrement
          page_size: "2"

        # page_size to be an interpolated string value
        pagination_strategy:
          type: OffsetIncrement
          page_size: "{{ parameters['items_per_page'] }}"

    Attributes:
        page_size (InterpolatedString): the number of records to request
    """

    config: Config
    page_size: Union[str, int]
    parameters: InitVar[Mapping[str, Any]]

    def __post_init__(self, parameters: Mapping[str, Any]):
        self._offset = 0
        page_size = str(self.page_size) if isinstance(self.page_size, int) else self.page_size
        self._page_size = InterpolatedString(page_size, parameters=parameters)

    def next_page_token(self, response: requests.Response, last_records: List[Mapping[str, Any]]) -> Optional[Any]:
        if len(last_records) < self._page_size.eval(self.config):
            return None
        else:
            self._offset += len(last_records)
            return self._offset

    def reset(self):
        self._offset = 0

    def get_page_size(self) -> Optional[int]:
        page_size = self._page_size.eval(self.config)
        if not isinstance(page_size, int):
            raise Exception(f"{page_size} is of type {type(page_size)}. Expected {int}")
        return page_size



#
# Copyright (c) 2023 Airbyte, Inc., all rights reserved.
#

import pytest
from airbyte_cdk.sources.declarative.create_partial import _key_is_unset_or_identical, create
from airbyte_cdk.sources.declarative.interpolation.interpolated_string import InterpolatedString


class AClass:
    def __init__(self, parameter, another_param, parameters):
        self.parameter = parameter
        self.another_param = another_param
        self.parameters = parameters


class OuterClass:
    def __init__(self, name, some_field, inner_param):
        self.name = name
        self.some_field = some_field
        self.inner_param = inner_param


class OuterOuterClass:
    def __init__(self, name, param, inner_class):
        self.name = name
        self.param = param
        self.inner_class = inner_class


def test_pass_parameter_to_create_function():
    object = create(AClass, parameter="A")(another_param="B")
    assert object.parameter == "A"
    assert object.another_param == "B"


def test_parameter_not_overwritten_by_parameters():
    object = create(AClass, parameter="A", another_param="B", **{"$parameters": {"parameter": "C"}})()
    assert object.parameter == "A"
    assert object.another_param == "B"


def test_overwrite_param():
    object = create(AClass, parameter="A", another_param="B")(parameter="C")
    assert object.parameter == "C"
    assert object.another_param == "B"


def test_string_interpolation():
    s = "{{ next_page_token['next_page_url'] }}"
    partial = create(InterpolatedString, string=s)
    interpolated_string = partial()
    assert interpolated_string.string == s


def test_string_interpolation_through_parameters():
    s = "{{ parameters['name'] }}"
    parameters = {"name": "airbyte"}
    partial = create(InterpolatedString, string=s, **parameters)
    interpolated_string = partial()
    assert interpolated_string.eval({}) == "airbyte"


def test_string_interpolation_through_parameters_keyword():
    s = "{{ parameters['name'] }}"
    parameters = {"$parameters": {"name": "airbyte"}}
    partial = create(InterpolatedString, string=s, **parameters)
    interpolated_string = partial()
    assert interpolated_string.eval({}) == "airbyte"


@pytest.mark.parametrize(
    "test_name, key, value, expected_result",
    [
        ("test", "key", "value", True),
        ("test", "key", "a_different_value", False),
        ("test", "a_different_key", "value", True),
    ],
)
def test_key_is_unset_or_identical(test_name, key, value, expected_result):
    mapping = {"key": "value"}
    result = _key_is_unset_or_identical(key, value, mapping)
    assert expected_result == result
